
from __future__ import print_function
from __future__ import division
import argparse
import subprocess
from codecs import open
import sys
import webbrowser
import os
import datetime

import aggr


__version__ = '0.1.1'
__author__ = 'kkka'
__license__ = 'MIT'

def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('cachedir', metavar='CACHE_DIR')
    behavior = parser.add_mutually_exclusive_group(required=True)

    behavior.add_argument('--daily', action='store_const', dest='type', const=aggr.daily_key, help=u'a day unit')
    behavior.add_argument('--monthly', action='store_const', dest='type', const=aggr.monthly_key, help=u'a month unit')
    behavior.add_argument('--weekly', action='store_const', dest='type', const=aggr.weekly_key, help=u'a week unit')

    br = parser.add_mutually_exclusive_group()
    br.add_argument('--no-browser', action='store_true', help=u"don't launch browser")
    br.add_argument('--force-browser', action='store_true', help=u"launch browser and don't gnuplot")

    parser.add_argument('--no-gnuplot', action='store_true', help=u"don't launch gnuplot")

    return parser

def main():
    parser = get_parser()
    args = parser.parse_args()

    fdata = 'data.txt'
    with open(fdata, 'w', encoding='utf8') as out:
        aggr.aggr_to_file(args.cachedir, tokey=args.type, out=out)

    gp = make_gpscript(fdata=fdata, type=args.type.name,
                       cache_size=aggr.size_all(args.cachedir))

    with open('data.gp', 'w', encoding='utf8') as fgp:
        fgp.write(gp)

    with open(os.devnull, 'w') as devnull:
        try:
            subprocess.call(['gnuplot', '--version'], stdout=devnull)
        except OSError as e:
            gnuplot = False
        else:
            gnuplot = True


    if not gnuplot:
        msg  = (u'open http://gnuplot.respawned.com/ ...\n'
                u'Please copipe data.txt to Data form.\n'
                u'Please copipe data.gp to Plot Script form.')
        print(msg)

    if (not args.no_browser
        and (args.force_browser or not gnuplot)):
        webbrowser.open('http://gnuplot.respawned.com/')
        sys.exit()

    if gnuplot and not args.no_gnuplot:
        ret = subprocess.call(['gnuplot', 'data.gp'])


def make_gpscript(fdata, type, cache_size):
    for u in ('B', 'KB', 'MB', 'GB', 'TB'):
        if cache_size <= 1024:
            size = '{:.2f}{}'.format(cache_size, u)
            break
        cache_size = cache_size / 1024


    with open(fdata, 'r', encoding='utf8') as f:
        dat = f.read()

    lines = [line.strip() for line in dat.splitlines() if len(line.strip()) != 0]
    first = lines[0]
    last = lines[-1]

    begin, _ = first.split(None, 1)
    end, _ = last.split(None, 1)

    if len(begin.split('/')) == 2:
        timefmt = '%Y/%m'
    elif len(begin.split('/')) == 3:
        timefmt = '%Y/%m/%d'
    else:
        raise RuntimeError('illegal time format')


    begin_d = datetime.datetime.strptime(begin, timefmt)
    end_d = datetime.datetime.strptime(end, timefmt)
    years = int((end_d - begin_d).total_seconds() // datetime.timedelta(days=365).total_seconds())
    if years <= 1:
        years = 1


    counts = [int(line.split(None, 1)[1]) for line in lines]
    maxcount = max(counts)
    ave = sum(counts) / len(counts)
    if maxcount > ave * 5 and ave != 0:
        ymax = ave * 5
    else:
        ymax = maxcount + 10

    gpformat = '''
set title 'shingetsu {type} posts {begin} - {end}({size})'
set xlabel 'month'
set ylabel 'post'

set timefmt '{timefmt}'
set xdata time
set format x '%m'
set xtics 60 * 60 * 24 * 30 * {years}

set xrange ['{begin}' : '{end}']
set yrange [0 : {ymax}]

set output 'out.svg'
set terminal svg
plot 'data.txt' using 1:2 with lines title 'post'
'''
    return gpformat.format(type=type, begin=begin, end=end,
                           size=size, timefmt=timefmt, ymax=ymax,
                           years=years)



if __name__ == '__main__':
    main()
